/* (non-javadoc)
 * BASEFileReader.java is part of BASEFile.
 *
 * BASEFile is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * BASEFile is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with BASEFile; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * johan.enell@onk.lu.se
 * Johan Enell, Dept Oncology, Lund University, S-221 85 Lund, Sweden
 */
package basefile;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.HashMap;
import java.util.Vector;

/**
 * Read sections from a file created from the database BASE. This class is
 * designed to read from System.in and will read section wise.
 * 
 * @author Johan Enell
 */
public class BASEFileReader
{

	/**
	 * The active baseFile.BASEFileSection that the reader is reading from.
	 */
	private BASEFileSection section = null;

	private LineNumberReader str;

	private boolean dataEnd;

	private boolean eof = false;

	private String buf;

	private File inFile;

	/**
	 * Create a new baseFile reader.
	 * 
	 * @param in
	 *            the stream from were the reader gets the data.
	 * @throws FileNotFoundException If the infile cant be found
	 */
	public BASEFileReader(File in) throws FileNotFoundException
	{
		inFile = in;
		str = new LineNumberReader(new FileReader(in));
		dataEnd = true;
	}

	/**
	 * Read the next baseFile section in the baseFile.
	 * 
	 * @return the section read.
	 * @throws BadSectionException
	 *             If the baseFile section is badly written
	 * @throws BadFormatException
	 *             If the baseFile is badly formatted (ie. first line is not
	 *             BASEFil)
	 * @throws IOException
	 *             If an I/O error occurs.
	 */
	public BASEFileSection readSection()
		throws BadSectionException, BadFormatException, IOException
	{
		return readSection(false);
	}

	/**
	 * Read the next baseFile section in the baseFile.
	 * 
	 * @param beginFile
	 *            Is set true if it is the begining of the baseFile.
	 * @return the section read or null if there is no more sections
	 * @throws BadSectionException
	 *             If the baseFile section is badly written
	 * @throws BadFormatException
	 *             If the baseFile is badly formatted (ie. first line is not
	 *             BASEFil)
	 * @throws IOException
	 *             If an I/O error occurs.
	 */
	public BASEFileSection readSection(boolean beginFile)
		throws BadFormatException, IOException, BadSectionException
	{
		if (section == null)
		{
			section = new BASEFileSection();
		}
		else
		{
			section.clear();
		}
		
		section.setStartLine(str.getLineNumber() + 1);
		skipData();
		section.setStartLine(str.getLineNumber() + 1);

		if (!section.setHeaders(readHeaders(beginFile)))
		{
			return null;
		}

		section.setDataStartLine(str.getLineNumber() + 1);

		String s = section.findStringOpt("section");
		if (s == null) return null;
		return section;
	}
	
	/**
	 * TODO: Write javadoc for readSpotSection()
	 * @param <R>
	 * @param <S>
	 * @return
	 * @throws BadFormatException 
	 * @throws IOException 
	 * @throws BadSectionException 
	 */
	@SuppressWarnings("unchecked")
	public <R, S> BASEFileSpotSection<R, S>  readSpotSection()
		throws BadFormatException, IOException, BadSectionException
	{
		section = new BASEFileSpotSection<R, S>();
		while (section != null && !section.isType("spots"))
		{
			section = readSection(false);
		}
		return (BASEFileSpotSection<R, S>) section;
	}

	public void resetDataPosition(BASEFileSection section)
	{
		try
		{
			str.close();
			str = new LineNumberReader(new FileReader(inFile));
			while (str.getLineNumber() < section.getDataStartLine())
			{
				str.readLine();
			}
			dataEnd = false;
		}
		catch (IOException e)
		{
			//Should not happend
			e.printStackTrace();
		}
	}

	/**
	 * This will read a singel line from the active section.
	 * 
	 * @return The data row as an String[]. Returns <code>null</code> if the
	 *         section end or the file end is reached.
	 * @throws IOException
	 *             If an I/O error occurs.
	 */
	public String[] readDataRow() throws IOException
	{
		Vector<String> vec = new Vector<String>();

		if (!readLine() || dataEnd) return null;

		int tok = buf.indexOf("\t"), otok = 0;

		while (tok < buf.length())
		{
			tok = buf.indexOf("\t", otok);
			if (tok == -1)
			{
				tok = buf.length();
			}
			vec.add(buf.substring(otok, tok));
			otok = tok + 1;
		}

		return vec.toArray(new String[vec.size()]);
	}

	/**
	 * This will read a singel line from the active section.
	 * 
	 * @param nbr
	 *            The number of fields that the row consists of
	 * @return The data row as an String[]. Returns <code>null</code> if the
	 *         section end or the file end is reached.
	 * @throws IOException
	 *             If an I/O error occurs.
	 * @throws TooFewFieldsException
	 *             If <code>nbr</code> is less than the number of fields in
	 *             the row.
	 * @throws TooManyFieldsException
	 *             If <code>nbr</code> is more than the number of fields in
	 *             the row.
	 */
	public String[] readDataRow(int nbr)
		throws IOException, TooFewFieldsException, TooManyFieldsException
	{
		String[] vec = new String[nbr];
		dataEnd = false;

		if (nbr < 0)
		{
			throw new TooFewFieldsException("Too few fields in " + section.getType() + ", line " + str.getLineNumber());
		}
		if (!readLine() || dataEnd) return null;

		int tok = buf.indexOf("\t"), otok = 0;

		while (tok < buf.length())
		{
			tok = buf.indexOf("\t", otok);
			if (tok == -1)
			{
				tok = buf.length();
			}
			vec[vec.length - nbr] = buf.substring(otok, tok);
			otok = tok + 1;
			nbr--;
		}

		if (nbr < 0)
		{
			throw new TooFewFieldsException("Too few fields in " + section.getType() + ", line " + str.getLineNumber());
		}
		if (nbr > 0)
		{
			throw new TooManyFieldsException("Too many fields in " + section.getType() + ", line " + str.getLineNumber());
		}
		return vec;
	}

	/**
	 * Read a line from the stream.
	 * 
	 * @return False if there is no more datarows in the section.
	 * @throws IOException
	 *             If an I/O error occurs.
	 */
	private boolean readLine() throws IOException
	{
		buf = str.readLine();
		if (buf == null)
		{
			dataEnd = true;
			eof = true;
			return false;
		}
		else if (buf.equals(""))
		{
			dataEnd = true;
			return false;
		}
		return true;
	}

	/**
	 * Skips all datarows so an new section can be read.
	 * 
	 * @return False if end of file is reached.
	 * @throws IOException
	 *             If an I/O error occurs.
	 */
	private boolean skipData() throws IOException
	{
		if (!dataEnd)
		{
			if (!readLine()) return false;
			while (!dataEnd)
			{
				if (!readLine()) return false;
			}
		}
		return true;
	}

	/**
	 * Reads the headers in a section.
	 * 
	 * @param begin
	 *            is true if it is the begining of the baseFile.
	 * @return A HashMap with the fields as keys.
	 * @throws IOException
	 *             If an I/O error occurs.
	 * @throws BadFormatException
	 *             If the baseFile is badly formatted (ie. first line is not
	 *             BASEFil)
	 */
	private HashMap<String, String> readHeaders(boolean begin)
		throws IOException, BadFormatException
	{
		HashMap<String, String> nameValue = new HashMap<String, String>();
		if (begin)
		{
			if (!readLine()) return null;
			if (!buf.startsWith("BASEfile"))
			{
				throw new BadFormatException("The file must start with 'BASEfile'");
			}
		}
		if (!readLine()) return null;
		while (buf != null && buf.equals(""))
			if (!readLine()) return null;
		dataEnd = false;
		while (buf != null && !buf.equals("%"))
		{
			int tab = buf.indexOf("\t");
			if (tab != -1)
			{
				nameValue.put(buf.substring(0, tab), buf.substring(tab + 1));
			}
			else if (buf.length() > 0) nameValue.put(buf, "");
			if (!readLine()) return null;
		}
		return nameValue;
	}

	/**
	 * @return Returns the section.
	 */
	public Object getSection()
	{
		return section;
	}

}