/*
 * (non-javadoc) BASEFileSection.java is part of BASEFile.
 * 
 * BASEFile is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * BASEFile is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * BASEFile; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * johan.enell@onk.lu.se Johan Enell, Dept Oncology, Lund University, S-221 85
 * Lund, Sweden
 */
package basefile;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * This class describes a section in a baseFile created from the BASE database.
 * 
 * @author Johan Enell
 */
/**
 * 
 * @author Johan Enell, johan.enell@onk.lu.se, Dept Oncology, Lund University,
 *         S-221 85 Lund, Sweden
 */
public class BASEFileSection
{

	/**
	 * The line number in the inputstream were the section starts
	 */
	private int startLine;

	/**
	 * The line number in the inpustream were the data in the section starts
	 */
	private int dataStartLine;

	/**
	 * The headers in the section put in a HashMap with the parameternames as
	 * keys.
	 */
	private final HashMap<String, String> headers = new HashMap<String, String>();

	/**
	 * Creates a new BASEFileSection.
	 */
	public BASEFileSection()
	{
		startLine = -1;
		dataStartLine = -1;
		setHeader("section", "");
	}

	/**
	 * Find the specified header from the headers and return it as an int.
	 * 
	 * @param name
	 *            The name of the header
	 * @return the option value as an int
	 * @throws NumberFormatException
	 *             If the option is not an integer
	 */
	public final Integer findIntOpt(String name)
		throws NumberFormatException
	{
		String ret = headers.get(name);
		try
		{
			return new Integer(ret);
		}
		catch (NumberFormatException e)
		{
			throw new NumberFormatException("The value of parameter " + name + " is not an int: " + ret);
		}
	}

	/**
	 * Find the specified header from the headers and return it as a float.
	 * 
	 * @param name
	 *            The name of the header
	 * @return the option value as a float
	 * @throws NumberFormatException
	 *             If the option is not a float
	 */
	public Float findFloatOpt(String name)
		throws NumberFormatException
	{
		String ret = headers.get(name);
		return new Float(ret);
	}

	/**
	 * Find the specified header from the headers and return it as a String.
	 * 
	 * @param name
	 *            The name of the header
	 * @return the option value as a String
	 */
	public final String findStringOpt(String name)
	{
		String ret = headers.get(name);
		return ret;
	}

	/**
	 * Find the specified header from the headers and return it as a String[].
	 * The values is separated by the tab (\u0009) character.
	 * 
	 * @param name
	 *            The name of the header
	 * @return the option value as a String
	 */
	public final String[] findStringOpts(String name)
	{
		return findStringOpts(name, "\t");
	}
	
	/**
	 * Find the specified header from the headers and return it as a String[].
	 * The values is separated by the del String.
	 * 
	 * @param name The name of the header
	 * @param del The string that splits the value. Using the method String.split()
	 * @return the option value as a String or null if name isnt found in the headers
	 */
	public final String[] findStringOpts(String name, String del)
	{
		String value = headers.get(name);
		if (value != null)
		{
			return value.split(del);
		}
		return null;
	}

	/**
	 * Find the specified header from the headers and return it as a boolean.
	 * 
	 * @param name
	 *            The name of the header
	 * @return the option value as a boolean
	 * @throws NumberFormatException
	 *             If the option is not a boolean
	 */
	public final boolean findBooleanOpt(String name)
	{
		String ret = headers.get(name);

		boolean value = false;
		try
		{
			value = Boolean.valueOf(ret).booleanValue();
		}
		catch (NumberFormatException e)
		{
			throw new NumberFormatException("The value of parameter " + name + " is not a boolean: " + ret);
		}
		return value;
	}

	/**
	 * Used to get the values from a header as a {@link java.util.List<String>}. 
	 * The order is the same as in the basefile if the user hasent changed it.
	 * 
	 * @param name
	 *            The name of the header
	 * @return A list of the values.
	 */
	public final List<String> findFieldList(String name)
	{
		String s = headers.get(name);
		if (s == null) return null;
		return new ArrayList<String>(Arrays.asList(s.split("\\t")));

	}

	/**
	 * Used to get the values from a header as a {@link java.util.List<Integer>}. 
	 * The order is the same as in the basefile if the user hasent changed it.
	 * 
	 * @param name
	 *            The name of the header
	 * @return A list of the values.
	 */
	public final List<Integer> findFieldIntList(String name)
	{
		String s = headers.get(name);
		if (s == null) return null;
		ArrayList<Integer> ret = new ArrayList<Integer>();
		for (String assay : Arrays.asList(s.split("\\t")))
		{
			ret.add(new Integer(assay));
		}
		return ret;
	}

	/**
	 * Puts a header in the header map. The header is stored as a String using
	 * the Object.toString() method. If the map previously contained a mapping
	 * for this key, the old value is replaced.
	 * 
	 * @param key
	 *            key with which the specified value is to be associated.
	 * @param value
	 *            value to be associated with the specified key.
	 * @return previous value associated with specified key, or null if there
	 *         was no mapping for key. A null return can also indicate that the
	 *         HashMap previously associated null with the specified key.
	 */
	public final String setHeader(Object key, Object value)
	{
		return headers.put(key.toString(), value.toString());
	}

	/**
	 * Puts a header in the header map. The header is stored as a String using
	 * the Object.toString() method. All the Object in the array is conbined the
	 * with a tabulation character '\t'. If the map previously contained a
	 * mapping for this key, the old value is replaced.
	 * 
	 * @param key
	 *            key with which the specified value is to be associated.
	 * @param value
	 *            values to be associated with the specified key.
	 * @return previous value associated with specified key, or null if there
	 *         was no mapping for key. A null return can also indicate that the
	 *         HashMap previously associated null with the specified key.
	 */
	public final String setHeader(Object key, Object... value)
	{
		return this.setHeader(key, '\t', value);
	}

	/**
	 * Puts a header in the header map. The header is stored as a String using
	 * the Object.toString() method. All the Object in the array is conbined the
	 * with the character c. If the map previously contained a mapping for this
	 * key, the old value is replaced.
	 * 
	 * @param key
	 *            key with which the specified value is to be associated.
	 * @param c
	 *            the character used to combine the value array.
	 * @param value
	 *            values to be associated with the specified key.
	 * @return previous value associated with specified key, or null if there
	 *         was no mapping for key. A null return can also indicate that the
	 *         HashMap previously associated null with the specified key.
	 */
	public final String setHeader(Object key, char c, Object... value)
	{
		StringBuffer s = new StringBuffer("");
		if (value.length > 0)
		{
			s = new StringBuffer(value[0].toString());
			for (int i = 1; i < value.length; i++)
			{
				s.append(c);
				s.append(value[i]);
			}
		}
		return this.setHeader(key, s);
	}

	public final String getHeader(Object key)
	{
		return this.headers.get(key.toString());
	}

	public final String removeHeader(Object key)
	{
		if (key.equals("section"))
		{
			return "";
		}
		return headers.remove(key.toString());
	}

	final boolean setHeaders(HashMap<String, String> headers)
	{
		if (headers == null)
		{
			return false;
		}
		this.headers.clear();
		for (String key : headers.keySet())
		{
			this.headers.put(key, headers.get(key));
		}
		return true;
	}

	/**
	 * Get the type of the section.
	 * 
	 * @return The value of the header 'section'
	 */
	public final String getType()
	{
		return headers.get("section");
	}

	/**
	 * @param string
	 * @return
	 */
	public final boolean isType(String string)
	{
		return getType().equals(string);
	}

	public final int getDataStartLine()
	{
		return dataStartLine;
	}

	public final void setDataStartLine(int dataStartLine)
	{
		this.dataStartLine = dataStartLine;
	}

	public final int getStartLine()
	{
		return startLine;
	}

	public final void setStartLine(int startLine)
	{
		this.startLine = startLine;
	}

	public void clear()
	{
		this.headers.clear();
		setHeader("section", "");
		startLine = -1;
		dataStartLine = -1;
	}

	@Override
	public String toString()
	{
		String ret = "section\t" + this.getType() + "\n";

		for (String key : headers.keySet())
		{
			if (!key.equals("section"))
			{
				ret += key + "\t" + headers.get(key) + "\n";
			}
		}
		ret += "%";

		return ret;
	}
}