/*
 $Id$

 Copyright (C) 2006 Johan Enell

 This file is part of BASE - BioArray Software Environment.
 Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 BASE is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 02111-1307, USA.
 */
package smooth;

import basefile.BASEFileException;

/**
 * The Reporter for the Smooth plug-in. It contains the position, reporter,
 * chromosome, basepair position and a link to its {@link Spot spot} data.
 * 
 * @author Johan Enell, johan.enell@med.lu.se, Dept Oncology, Lund University
 */
public class Reporter
{
	private final int position;

	private final int reporter;

	private final int chromosome;

	private final int chrPosition;

	private final Spot spot;

	/**
	 * Constructs a new Reporter. Chromosome X and Y are translated to 23 resp
	 * 24. The start and end are used to calculate the chromosome position, (end -
	 * start) / 2.
	 * 
	 * @param position
	 *        the position on the assay
	 * @param reporter
	 *        the id of the reporter
	 * @param chromosome
	 *        the chromosome number
	 * @param start
	 *        the basepair start position on its chromoeome
	 * @param end
	 *        the basepair end position on its chromoeome
	 * @param spot
	 *        the spotdata for this reporter
	 * @throws BASEFileException
	 *         if any parameter cant be parsed as an integer or if chromosome
	 *         position is less then 1
	 */
	public Reporter(String position, String reporter, String chromosome, String start, String end, Spot spot)
		throws BASEFileException
	{
		try
		{
			this.spot = spot;
			this.position = Integer.parseInt(position);
			this.reporter = Integer.parseInt(reporter);

			int s = Integer.parseInt(start);
			int e = Integer.parseInt(end);
			this.chrPosition = (e - s) / 2 + s;
			if (chrPosition < 1) throw new BASEFileException("Chromosome position must be greater then 0");

			if (chromosome.equalsIgnoreCase("X")) this.chromosome = 23;
			else if (chromosome.equalsIgnoreCase("Y")) this.chromosome = 24;
			else this.chromosome = Integer.parseInt(chromosome);
		}
		catch (NumberFormatException e)
		{
			throw new BASEFileException(e.getMessage());
		}

	}

	/**
	 * Gets the chromosome of this reporter. X is 23 and Y is 24
	 * 
	 * @return the chromosome
	 */
	public final int getChromosome()
	{
		return chromosome;
	}

	/**
	 * Gets the assay position of this reporter.
	 * 
	 * @return the position
	 */
	public final int getPosition()
	{
		return position;
	}

	/**
	 * Gets the reporter id of this reporter.
	 * 
	 * @return the reporter id
	 */
	public final int getReporter()
	{
		return reporter;
	}

	/**
	 * Gets the basepair position on the chromosome of this reporter.
	 * 
	 * @return the chromosome position
	 */
	public final int getChrPosition()
	{
		return chrPosition;
	}

	/**
	 * Gets the spot data for this reporter.
	 * 
	 * @return the spot
	 */
	public final Spot getSpot()
	{
		return spot;
	}

	@Override
	public String toString()
	{
		return position + "\t" + reporter;
	}

}
