package smooth;

import basefile.BASEFileSpotSection;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * This class computes a smooth M for every spot using a sliding window across
 * the chromosomes. The window size can either be defined as number of spots or
 * number of basepairs. I spots are choosen then the window will have a constant
 * size but the distance on the chromosome between the fist and last spot can
 * differ a lot. On the other hand, if basepairs are choosen the number of spots
 * can differ but the distance on the chromosome will never be greater than the
 * window size value. The spot can aither be smoothed against a mean or a median
 * and the impact of the smooth can be adjusted using the weight parameter. A
 * weight value of 0 means that the spot wont be smoothed at all and a value of
 * 1 means the spot will be smoothed the while distance. Calculation:
 * 
 * <pre>
 *     M' = M + w * (a - M)
 * </pre>
 * 
 * Where M' is the new M and M is the old, w is the weight and a is the average
 * of the window.
 * 
 * @author Johan Enell, johan.enell@med.lu.se, Dept Oncology, Lund University
 */
class Smooth
{
	private String winType;

	private String average;

	private int winSize;

	private float w;

	/**
	 * Constructs a new smoother with a set of parameters.
	 * 
	 * @param winType
	 *        the type of window. Can either be "cl" for clones or "bp"
	 *        basepairs.
	 * @param average
	 *        the average method to use. Can either be "median" or "mean".
	 * @param winSize
	 *        the size of the window. The interpretation is determined of the
	 *        window type.
	 * @param w
	 *        the weight
	 */
	Smooth(String winType, String average, int winSize, float w)
	{
		this.winType = winType;
		this.average = average;
		this.winSize = winSize;
		this.w = w;
	}

	/**
	 * This will smooth the spots in the section and store the smoothed value in
	 * {@link Spot#setMprim(float)}. The method will start by sorting the
	 * reporters in the section by chromosome and each chromosome by chromosome
	 * position. Then it will iterate over the chromosome and slide a window
	 * over each.
	 * 
	 * @param bfss
	 *        the spot section that is going to be smoothed.
	 */
	void compute(BASEFileSpotSection<Reporter, Spot> bfss)
	{
		ChromosomeIterator chromosomeIterator = new ChromosomeIterator(bfss);
		bfss.sortReporter(new Comparator<Reporter>()
		{
			public int compare(Reporter r1, Reporter r2)
			{
				if (r1.getChromosome() < r2.getChromosome()) return -1;
				else if (r1.getChromosome() == r2.getChromosome())
				{
					if (r1.getChrPosition() < r2.getChrPosition()) return -1;
					else if (r1.getChrPosition() == r2.getChrPosition()) return 0;
					else if (r1.getChrPosition() > r2.getChrPosition()) return 1;
				}
				return 1;
			}
		});

		while (chromosomeIterator.hasNext())
		{
			List<Reporter> reporters = chromosomeIterator.next();
			LinkedList<Float> window = new LinkedList<Float>();

			// Start and stop index for window include, exclude
			// [start, stop)
			int start = 0;
			int stop = 0;

			for (Reporter r : reporters)
			{
				if (winType.equals("cl"))
				{
					if (stop < reporters.size())
					{
						if (!window.isEmpty())
						{
							window.removeLast();
							start++;
						}
						while (stop < reporters.size() && window.size() < winSize)
						{
							window.addFirst(reporters.get(stop).getSpot().getM());
							stop++;
						}
					}
				}
				else if (winType.equals("bp"))
				{
					if (stop < reporters.size())
					{
						while (!window.isEmpty() && r.getChrPosition() - reporters.get(start).getChrPosition() > winSize / 2)
						{
							window.removeFirst();
							start++;
						}
						while (stop < reporters.size() && reporters.get(stop).getChrPosition() - reporters.get(start).getChrPosition() < winSize)
						{
							window.addFirst(reporters.get(stop).getSpot().getM());
							stop++;
						}
					}
				}
				float mean = Float.NaN;
				if (average.equals("median"))
				{
					mean = median(window.toArray(new Float[window.size()]));
				}
				else if (average.equals("mean"))
				{
					mean = mean(window.toArray(new Float[window.size()]));
				}

				if (!Float.isNaN(mean))
				{
					float fit = w * (mean - r.getSpot().getM());
					r.getSpot().setMprim(r.getSpot().getM() + fit);
				}
			}
		}
	}

	/**
	 * This will calculare the mean of an array.
	 * 
	 * @param floats
	 *        the array to calculate the mean
	 * @return the mean or Float.NaN if the mean cant be calculated
	 */
	private float mean(Float[] floats)
	{
		float sum = 0;
		if (floats.length == 0)
		{
			return Float.NaN;
		}
		else
		{
			for (int i = 0; i < floats.length; i++)
			{
				sum += floats[i];
			}
		}
		return sum / floats.length;
	}

	/**
	 * This will calculate the median of an array.
	 * 
	 * @param floats
	 *        the array to calculate the median
	 * @return the median or Float.NaN if the median cant be calculated
	 */
	private float median(Float[] floats)
	{
		if (floats.length < 3)
		{
			return mean(floats);
		}
		else
		{
			Arrays.sort(floats);
			int i = (int) Math.ceil((floats.length - 1) / 2);
			if (floats.length % 2 == 1)
			{
				return floats[i];
			}
			return floats[i] + floats[i + 1] / 2;
		}
	}

	/**
	 * This iterator will over chromosome in a spot section. The reporters must
	 * be sorted by chromosome for this iterator to work correct.
	 * 
	 * @author Johan Enell, johan.enell@med.lu.se, Dept Oncology, Lund
	 *         University
	 */
	private class ChromosomeIterator implements Iterator<List<Reporter>>
	{
		private int start = 0;

		private int end = 0;

		private int currentChr = -1;

		private final BASEFileSpotSection<Reporter, Spot> bfss;

		public ChromosomeIterator(BASEFileSpotSection<Reporter, Spot> bfss)
		{
			this.bfss = bfss;
		}

		public boolean hasNext()
		{
			start = end;
			if (start >= bfss.getReporterSize())
			{
				return false;
			}
			currentChr = bfss.getReporter(start).getChromosome();
			while (end < bfss.getReporterSize())
			{
				Reporter r = bfss.getReporter(end);
				if (r.getChromosome() != currentChr)
				{
					break;
				}
				end++;
			}
			return true;
		}

		public List<Reporter> next()
		{
			if (start >= bfss.getHeight())
			{
				return null;
			}
			return bfss.reporterSubList(start, end);
		}

		public void remove()
		{
			throw new UnsupportedOperationException();
		}

		public final int getCurrentChr()
		{
			return currentChr;
		}

	}
}
