/*
 $Id$

 Copyright (C) 2006 Johan Enell

 This file is part of BASE - BioArray Software Environment.
 Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 BASE is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 02111-1307, USA.
 */
package smooth;

import basefile.BASEFileException;
import basefile.BASEFileReader;
import basefile.BASEFileSection;
import basefile.BASEFileSpotSection;

import java.io.File;
import java.io.PrintStream;
import java.util.List;

/**
 * This is the main class for the smooth plugin.
 * 
 * @author Johan Enell, johan.enell@med.lu.se, Dept Oncology, Lund University
 */
public class SmoothMain
{

	/**
	 * The main method will parse the basefile. The first section must be named
	 * "settings" and the requierd parameters will be extracted and validated
	 * and a new smooth object will be constructed from these. Every following
	 * spot section will be computed using the smoother and written to the file
	 * stdout.txt.
	 * 
	 * @param args either empty or the name of the basefile
	 */
	public static void main(String[] args)
	{
		try
		{
			BASEFileReader bfr;
			if (args.length == 1) bfr = new BASEFileReader(new File(args[0]));
			else bfr = new BASEFileReader(new File("stdin.txt"));
			System.setOut(new PrintStream("stdout.txt"));

			BASEFileSection bfs = bfr.readSection(true);
			if (!bfs.isType("settings"))
			{
				throw new BASEFileException("Can't find section settings");
			}

			String wType = bfs.findStringOpt("w_type");
			String average = bfs.findStringOpt("average");
			int wSize = bfs.findIntOpt("w_size");
			float w = bfs.findFloatOpt("w");

			if (wSize <= 0)
			{
				System.err.println("Can't do anything with windowsize " + wSize);
				System.exit(-1);
			}
			if (w <= 0 || w > 1)
			{
				System.err.println("Weight must be greater than 0 and less or equal with 1");
				System.exit(-1);
			}
			System.out.println("BASEfile");

			Smooth smooth = new Smooth(wType, average, wSize, w);
			BASEFileSpotSection<Reporter, Spot> bfss = bfr.readSpotSection();
			while (bfss != null)
			{
				bfss.setDataMatrix(bfss.findIntOpt("count"), bfss.findFieldList("assays").size());
				List<String> columns = bfss.findFieldList("columns");
				List<String> assayFields = bfss.findFieldList("assayFields");

				int posCol = columns.indexOf("position");
				int repCol = columns.indexOf("reporter");
				int chrCol = columns.indexOf("chromosome");
				int startCol = columns.indexOf("startPosition");
				int endCol = columns.indexOf("endPosition");
				int dataCol = columns.indexOf("assayData");
				int i1Col = assayFields.indexOf("intensity1") + dataCol;
				int i2Col = assayFields.indexOf("intensity2") + dataCol;

				bfss.setDataMatrix(bfss.findIntOpt("count"), 1);

				String[] data = bfr.readDataRow();
				while (data != null)
				{
					try
					{
						Spot s = new Spot(data[i1Col], data[i2Col]);
						Reporter r = new Reporter(data[posCol], data[repCol], data[chrCol], data[startCol], data[endCol], s);
						bfss.addData(r, s);
					}
					catch (NumberFormatException e)
					{}
					data = bfr.readDataRow(7);
				}

				smooth.compute(bfss);

				bfss.setHeader("columns", "position", "reporter", "assayData");
				bfss.setHeader("assayFields", "l2ratio1_2", "l10intgmean1_2", "JE_smoothdiff");
				bfss.setHeader("setExtraFloats", "JE_smoothdiff");
				System.out.println(bfss);
				for (int i = 0; i < bfss.getReporterSize(); i++)
				{
					Reporter r = bfss.getReporter(i);
					System.out.println(r + "\t" + r.getSpot());
				}
				System.out.println();
				bfss = bfr.readSpotSection();
			}

		}
		catch (OutOfMemoryError oome)
		{
			System.err.println("No more memory in the java virtual machine. Start the application with more memory. If you need even more memory, duplicate the digit");
			System.exit(-1);
		}
		catch (Exception e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
	}

}
