/*
 $Id$

 Copyright (C) 2006 Johan Enell

 This file is part of BASE - BioArray Software Environment.
 Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 BASE is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 02111-1307, USA.
 */
package smooth;

import basefile.BASEFileException;

/**
 * The Spot for the smooth plugin. It contains intensity1, intensity2, M and the
 * new M.
 * 
 * @author Johan Enell, johan.enell@med.lu.se, Dept Oncology, Lund University
 */
public class Spot
{
	private final float int1;

	private final float int2;

	private final float m;

	private float mPrim;

	/**
	 * Constructs a spot from intensity1 and intensity2. It calculates the M
	 * value.
	 * 
	 * @param int1
	 *        intensity for channel 1
	 * @param int2
	 *        intensity for channel 2
	 * @throws BASEFileException
	 *         if the string cant be parsed as floats
	 */
	public Spot(String int1, String int2)
		throws BASEFileException
	{
		try
		{
			this.int1 = Float.parseFloat(int1);
			this.int2 = Float.parseFloat(int2);
			this.m = (float) (Math.log(this.int1 / this.int2) / Math.log(2));
		}
		catch (NumberFormatException e)
		{
			throw new BASEFileException(e.getMessage());
		}
	}

	/**
	 * Gets the intensity for channel 1.
	 * 
	 * @return intensity for channel 1
	 */
	public final float getInt1()
	{
		return int1;
	}

	/**
	 * Gets the intensity for channel 2.
	 * 
	 * @return intensity for channel 2
	 */
	public final float getInt2()
	{
		return int2;
	}

	/**
	 * Gets the M value calculated from the intensity values. log2(int1 / int2)
	 * 
	 * @return the M value
	 */
	public float getM()
	{
		return m;
	}

	/**
	 * Gets the new M value.
	 * 
	 * @return the new M
	 */
	public float getMprim()
	{
		return mPrim;
	}

	/**
	 * Gets the A value calculated from the intensity values. A=log10(sqrt(int1 *
	 * int2)).
	 * 
	 * @return the A value
	 */
	public float getA()
	{
		return (float) Math.log10(Math.sqrt(int1 * int2));
	}

	/**
	 * Sets the new M value.
	 * 
	 * @param m
	 *        the new M value
	 */
	public void setMprim(float m)
	{
		this.mPrim = m;
	}

	@Override
	public String toString()
	{
		return getMprim() + "\t" + getA() + "\t" + (getMprim() - getM());
	}
}
